var Crypto, Fs, Mime, Path, Util, empty_cb, etag_matches, methods, not_modified;
Util = require('util');
Path = require('path');
Fs = require('fs');
Crypto = require('crypto');
Mime = require('simple-mime')('application/octect-stream');
methods = ['GET', 'HEAD'];
module.exports = function(root, options) {
  var defaultFile;
  if (options == null) {
    options = {};
  }
  root = Path.normalize(root);
  defaultFile = options["default"];
  delete options["default"];
  if (!(options.etag != null)) {
    options.etag = module.exports.etag;
  }
  return function(req, res, next) {
    if (methods.indexOf(req.method) === -1) {
      res.writeHead(405, {
        accept: 'GET, HEAD'
      });
      return res.end();
    }
    if (defaultFile && req.url.match(/\/$/)) {
      req.url = "" + req.url + defaultFile;
    }
    options.file = "" + root + req.url;
    return module.exports.serve(req, res, options, function(err) {
      if (err) {
        return next();
      }
    });
  };
};
module.exports.serve = function(req, res, options, next) {
  var file;
  if (options == null) {
    options = {};
  }
  file = options.file;
  delete options.file;
  return Fs.stat(file, function(err, stats) {
    if (err) {
      if (next) {
        next(err);
      }
      return;
    }
    return module.exports.writeHead(req, res, file, stats, options, function(status) {
      if (status < 200 || status > 299 || req.method === 'HEAD') {
        res.end();
        if (next) {
          return next();
        }
      } else {
        return Util.pump(Fs.createReadStream(file), res, next || empty_cb);
      }
    });
  });
};
empty_cb = function() {};
module.exports.writeHead = function(req, res, file, stats, options, cb) {
  var headers, ifmod, ifunmod, since, status;
  if (options == null) {
    options = {};
  }
  headers = module.exports.defaultHeaders(file, stats);
  status = options.status || 200;
  ifmod = req.headers['if-modified-since'];
  if (ifmod) {
    since = Date.parse(ifmod);
    if (since && since > stats.mtime.getTime()) {
      return not_modified(res, headers, cb);
    }
  }
  ifunmod = req.headers['if-unmodified-since'];
  if (ifunmod) {
    since = Date.parse(ifunmod);
    if (since && since < stats.mtime.getTime()) {
      headers['content-length'] = 0;
      res.writeHead(412, headers);
      return cb(412);
    }
  }
  if (options.etag) {
    return options.etag(file, stats, function(etag) {
      headers.etag = etag;
      if (etag_matches(req.headers['if-none-match'], etag)) {
        return not_modified(res, headers, cb);
      }
      res.writeHead(status, headers);
      return cb(status);
    });
  } else {
    res.writeHead(status, headers);
    return cb(status);
  }
};
module.exports.defaultHeaders = function(file, stats) {
  return {
    'content-type': Mime(file),
    'content-length': stats.size,
    'last-modified': stats.mtime.toUTCString()
  };
};
module.exports.etag = function(file, stats, next) {
  var hash;
  hash = Crypto.createHash('sha1');
  hash.update(stats.ino);
  hash.update(stats.size);
  hash.update(stats.mtime);
  return next("\"" + (hash.digest('hex')) + "\"");
};
not_modified = function(res, headers, cb) {
  headers['content-length'] = 0;
  res.writeHead(304, headers);
  return cb(304);
};
etag_matches = function(request_etag, file_etag) {
  return (request_etag != null) && (file_etag != null) && request_etag === file_etag;
};